#include <QtUiTools/QUiLoader>
#include <QtCore/QStringList>
#include <QtCore/QFile>
#include <QtGui/QApplication>
#include <QtGui/QWidget>

#include "uiutils.h"

void UiUtils::install( QScriptEngine *engine )
{
    installWidgets( engine );

    QScriptValue globalObject = engine->globalObject();
    QScriptValue utilsObject = engine->newObject();
    globalObject.setProperty( "UiUtils", utilsObject );

    QScriptValue fun = engine->scriptValue( UiUtils::load );
    utilsObject.setProperty("load", fun);
    fun = engine->scriptValue( UiUtils::exec );
    utilsObject.setProperty("exec", fun);
}

void UiUtils::installWidgets( QScriptEngine *engine )
{
    QScriptValue globalObject = engine->globalObject();
    QUiLoader loader;

    QStringList widgets = loader.availableWidgets();
    for ( int i=0; i < widgets.size(); ++i ) {
	QScriptValue fun = engine->scriptValue( createWidget );
	fun.setProperty( QString("functionName"), engine->scriptValue(widgets[i]) );

	globalObject.setProperty(widgets[i], fun);
    }
}

QScriptValue UiUtils::createWidget(QScriptContext *context, QScriptEngine *engine)
{
    if ( context->argumentCount() > 1 )
	return context->throwError("Load takes one argument");

    QWidget *parent = 0;
    if ( context->argumentCount() ) {
	QScriptValue parentVal = context->argument(0);
	if ( !parentVal.isQObject() )
	    return context->throwError("The argument to load should be a QWidget");

	QObject *parentObj = parentVal.toQObject();
	parent = qobject_cast<QWidget *>(parentObj);
	if ( !parent )
	    return context->throwError("The argument to load should be a QWidget");
    }

    QString self = context->callee().property( "functionName" ).toString();
    QUiLoader loader;
    QWidget *w = loader.createWidget( self, parent );
    return engine->scriptValueFromQObject( w );
}

QScriptValue UiUtils::load(QScriptContext *context, QScriptEngine *engine)
{
    if ( context->argumentCount() != 1 ) {
	return context->throwError("Load takes one argument");
    }

    QUiLoader loader;
    QFile f( context->argument(0).toString() );
    f.open(QIODevice::ReadOnly);
    QWidget *w = loader.load( &f );

    return engine->scriptValueFromQObject( w );
}

QScriptValue UiUtils::exec(QScriptContext *context, QScriptEngine *engine)
{
    if ( context->argumentCount() )
	return context->throwError("Load takes no arguments");

    return engine->scriptValue( QApplication::exec() );
}
